/*
    Project: TCTEMP
    Version: V1.7
    File   : tc_des.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tc.h"

#ifdef DES

#define WEAK_DES_KEYS 64
//#define  DES_KEYS_WITH_PARITY_BITS

#if WEAK_DES_KEYS == 16

/*
 * Weak and semiweak keys as specified by FIPS PUB 74 and
 * Bruce Schneier, "Applied Cryptography. Protocols, Algorithms and Source
 * Code in C." John Wiley & Sons Inc, January 1996
 * http://en.wikipedia.org/wiki/Special:Booksources/0471128457
 */

#  ifdef DES_KEYS_WITH_PARITY_BITS

const unsigned __int8 WeakDESKeys[WEAK_DES_KEYS][8] = {
    { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
    { 0x01, 0x1f, 0x01, 0x1f, 0x01, 0x0e, 0x01, 0x0e },
    { 0x01, 0xe0, 0x01, 0xe0, 0x01, 0xf1, 0x01, 0xf1 },
    { 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe },
    { 0x1f, 0x01, 0x1f, 0x01, 0x0e, 0x01, 0x0e, 0x01 },
    { 0x1f, 0x1f, 0x1f, 0x1f, 0x0e, 0x0e, 0x0e, 0x0e },
    { 0x1f, 0xe0, 0x1f, 0xe0, 0x0e, 0xf1, 0x0e, 0xf1 },
    { 0x1f, 0xfe, 0x1f, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe },
    { 0xe0, 0x01, 0xe0, 0x01, 0xf1, 0x01, 0xf1, 0x01 },
    { 0xe0, 0x1f, 0xe0, 0x1f, 0xf1, 0x0e, 0xf1, 0x0e },
    { 0xe0, 0xe0, 0xe0, 0xe0, 0xf1, 0xf1, 0xf1, 0xf1 },
    { 0xe0, 0xfe, 0xe0, 0xfe, 0xf1, 0xfe, 0xf1, 0xfe },
    { 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01 },
    { 0xfe, 0x1f, 0xfe, 0x1f, 0xfe, 0x0e, 0xfe, 0x0e },
    { 0xfe, 0xe0, 0xfe, 0xe0, 0xfe, 0xf1, 0xfe, 0xf1 },
    { 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe }
};

#  else

static const unsigned __int8 pWeakDESKeys[WEAK_DES_KEYS][8] = {
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
    { 0x00, 0x1e, 0x00, 0x1e, 0x00, 0x0e, 0x00, 0x0e },
    { 0x00, 0xe0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf0 },
    { 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe },
    { 0x1e, 0x00, 0x1e, 0x00, 0x0e, 0x00, 0x0e, 0x00 },
    { 0x1e, 0x1e, 0x1e, 0x1e, 0x0e, 0x0e, 0x0e, 0x0e },
    { 0x1e, 0xe0, 0x1e, 0xe0, 0x0e, 0xf0, 0x0e, 0xf0 },
    { 0x1e, 0xfe, 0x1e, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe },
    { 0xe0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf0, 0x00 },
    { 0xe0, 0x1e, 0xe0, 0x1e, 0xf0, 0x0e, 0xf0, 0x0e },
    { 0xe0, 0xe0, 0xe0, 0xe0, 0xf0, 0xf0, 0xf0, 0xf0 },
    { 0xe0, 0xfe, 0xe0, 0xfe, 0xf0, 0xfe, 0xf0, 0xfe },
    { 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00 },
    { 0xfe, 0x1e, 0xfe, 0x1e, 0xfe, 0x0e, 0xfe, 0x0e },
    { 0xfe, 0xe0, 0xfe, 0xe0, 0xfe, 0xf0, 0xfe, 0xf0 },
    { 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe }
};

#  endif

#else if WEAK_DES_KEYS == 64

/*
 * Weak, semiweak and possibly weak keys as specified by
 * Bruce Schneier, "Applied Cryptography. Protocols, Algorithms and Source
 * Code in C." John Wiley & Sons Inc, January 1996
 * http://en.wikipedia.org/wiki/Special:Booksources/0471128457
 */

#  ifdef DES_KEYS_WITH_PARITY_BITS

const unsigned __int8 WeakDESKeys[WEAK_DES_KEYS][8] = {
    { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
    { 0x01, 0x01, 0x1f, 0x1f, 0x01, 0x01, 0x0e, 0x0e },
    { 0x01, 0x01, 0xe0, 0xe0, 0x01, 0x01, 0xf1, 0xf1 },
    { 0x01, 0x01, 0xfe, 0xfe, 0x01, 0x01, 0xfe, 0xfe },
    { 0x01, 0x1f, 0x01, 0x1f, 0x01, 0x0e, 0x01, 0x0e },
    { 0x01, 0x1f, 0x1f, 0x01, 0x01, 0x0e, 0x0e, 0x01 },
    { 0x01, 0x1f, 0xe0, 0xfe, 0x01, 0x0e, 0xf1, 0xfe },
    { 0x01, 0x1f, 0xfe, 0xe0, 0x01, 0x0e, 0xfe, 0xf1 },
    { 0x01, 0xe0, 0x01, 0xe0, 0x01, 0xf1, 0x01, 0xf1 },
    { 0x01, 0xe0, 0x1f, 0xfe, 0x01, 0xf1, 0x0e, 0xfe },
    { 0x01, 0xe0, 0xe0, 0x01, 0x01, 0xf1, 0xf1, 0x01 },
    { 0x01, 0xe0, 0xfe, 0x1f, 0x01, 0xf1, 0xfe, 0x0e },
    { 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe },
    { 0x01, 0xfe, 0x1f, 0xe0, 0x01, 0xfe, 0x0e, 0xf1 },
    { 0x01, 0xfe, 0xe0, 0x1f, 0x01, 0xfe, 0xf1, 0x0e },
    { 0x01, 0xfe, 0xfe, 0x01, 0x01, 0xfe, 0xfe, 0x01 },
    { 0x1f, 0x01, 0x01, 0x1f, 0x0e, 0x01, 0x01, 0x0e },
    { 0x1f, 0x01, 0x1f, 0x01, 0x0e, 0x01, 0x0e, 0x01 },
    { 0x1f, 0x01, 0xe0, 0xfe, 0x0e, 0x01, 0xf1, 0xfe },
    { 0x1f, 0x01, 0xfe, 0xe0, 0x0e, 0x01, 0xfe, 0xf1 },
    { 0x1f, 0x1f, 0x01, 0x01, 0x0e, 0x0e, 0x01, 0x01 },
    { 0x1f, 0x1f, 0x1f, 0x1f, 0x0e, 0x0e, 0x0e, 0x0e },
    { 0x1f, 0x1f, 0xe0, 0xe0, 0x0e, 0x0e, 0xf1, 0xf1 },
    { 0x1f, 0x1f, 0xfe, 0xfe, 0x0e, 0x0e, 0xfe, 0xfe },
    { 0x1f, 0xe0, 0x01, 0xfe, 0x0e, 0xf1, 0x01, 0xfe },
    { 0x1f, 0xe0, 0x1f, 0xe0, 0x0e, 0xf1, 0x0e, 0xf1 },
    { 0x1f, 0xe0, 0xe0, 0x1f, 0x0e, 0xf1, 0xf1, 0x0e },
    { 0x1f, 0xe0, 0xfe, 0x01, 0x0e, 0xf1, 0xfe, 0x01 },
    { 0x1f, 0xfe, 0x01, 0xe0, 0x0e, 0xfe, 0x01, 0xf1 },
    { 0x1f, 0xfe, 0x1f, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe },
    { 0x1f, 0xfe, 0xe0, 0x01, 0x0e, 0xfe, 0xf1, 0x01 },
    { 0x1f, 0xfe, 0xfe, 0x1f, 0x0e, 0xfe, 0xfe, 0x0e },
    { 0xe0, 0x01, 0x01, 0xe0, 0xf1, 0x01, 0x01, 0xf1 },
    { 0xe0, 0x01, 0x1f, 0xfe, 0xf1, 0x01, 0x0e, 0xfe },
    { 0xe0, 0x01, 0xe0, 0x01, 0xf1, 0x01, 0xf1, 0x01 },
    { 0xe0, 0x01, 0xfe, 0x1f, 0xf1, 0x01, 0xfe, 0x0e },
    { 0xe0, 0x1f, 0x01, 0xfe, 0xf1, 0x0e, 0x01, 0xfe },
    { 0xe0, 0x1f, 0x1f, 0xe0, 0xf1, 0x0e, 0x0e, 0xf1 },
    { 0xe0, 0x1f, 0xe0, 0x1f, 0xf1, 0x0e, 0xf1, 0x0e },
    { 0xe0, 0x1f, 0xfe, 0x01, 0xf1, 0x0e, 0xfe, 0x01 },
    { 0xe0, 0xe0, 0x01, 0x01, 0xf1, 0xf1, 0x01, 0x01 },
    { 0xe0, 0xe0, 0x1f, 0x1f, 0xf1, 0xf1, 0x0e, 0x0e },
    { 0xe0, 0xe0, 0xe0, 0xe0, 0xf1, 0xf1, 0xf1, 0xf1 },
    { 0xe0, 0xe0, 0xfe, 0xfe, 0xf1, 0xf1, 0xfe, 0xfe },
    { 0xe0, 0xfe, 0x01, 0x1f, 0xf1, 0xfe, 0x01, 0x0e },
    { 0xe0, 0xfe, 0x1f, 0x01, 0xf1, 0xfe, 0x0e, 0x01 },
    { 0xe0, 0xfe, 0xe0, 0xfe, 0xf1, 0xfe, 0xf1, 0xfe },
    { 0xe0, 0xfe, 0xfe, 0xe0, 0xf1, 0xfe, 0xfe, 0xf1 },
    { 0xfe, 0x01, 0x01, 0xfe, 0xfe, 0x01, 0x01, 0xfe },
    { 0xfe, 0x01, 0x1f, 0xe0, 0xfe, 0x01, 0x0e, 0xf1 },
    { 0xfe, 0x01, 0xe0, 0x1f, 0xfe, 0x01, 0xf1, 0x0e },
    { 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01 },
    { 0xfe, 0x1f, 0x01, 0xe0, 0xfe, 0x0e, 0x01, 0xf1 },
    { 0xfe, 0x1f, 0x1f, 0xfe, 0xfe, 0x0e, 0x0e, 0xfe },
    { 0xfe, 0x1f, 0xe0, 0x01, 0xfe, 0x0e, 0xf1, 0x01 },
    { 0xfe, 0x1f, 0xfe, 0x1f, 0xfe, 0x0e, 0xfe, 0x0e },
    { 0xfe, 0xe0, 0x01, 0x1f, 0xfe, 0xf1, 0x01, 0x0e },
    { 0xfe, 0xe0, 0x1f, 0x01, 0xfe, 0xf1, 0x0e, 0x01 },
    { 0xfe, 0xe0, 0xe0, 0xfe, 0xfe, 0xf1, 0xf1, 0xfe },
    { 0xfe, 0xe0, 0xfe, 0xe0, 0xfe, 0xf1, 0xfe, 0xf1 },
    { 0xfe, 0xfe, 0x01, 0x01, 0xfe, 0xfe, 0x01, 0x01 },
    { 0xfe, 0xfe, 0x1f, 0x1f, 0xfe, 0xfe, 0x0e, 0x0e },
    { 0xfe, 0xfe, 0xe0, 0xe0, 0xfe, 0xfe, 0xf1, 0xf1 },
    { 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe }
};

#  else

static const unsigned __int8 pWeakDESKeys[WEAK_DES_KEYS][8] = {
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
    { 0x00, 0x00, 0x1e, 0x1e, 0x00, 0x00, 0x0e, 0x0e },
    { 0x00, 0x00, 0xe0, 0xe0, 0x00, 0x00, 0xf0, 0xf0 },
    { 0x00, 0x00, 0xfe, 0xfe, 0x00, 0x00, 0xfe, 0xfe },
    { 0x00, 0x1e, 0x00, 0x1e, 0x00, 0x0e, 0x00, 0x0e },
    { 0x00, 0x1e, 0x1e, 0x00, 0x00, 0x0e, 0x0e, 0x00 },
    { 0x00, 0x1e, 0xe0, 0xfe, 0x00, 0x0e, 0xf0, 0xfe },
    { 0x00, 0x1e, 0xfe, 0xe0, 0x00, 0x0e, 0xfe, 0xf0 },
    { 0x00, 0xe0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf0 },
    { 0x00, 0xe0, 0x1e, 0xfe, 0x00, 0xf0, 0x0e, 0xfe },
    { 0x00, 0xe0, 0xe0, 0x00, 0x00, 0xf0, 0xf0, 0x00 },
    { 0x00, 0xe0, 0xfe, 0x1e, 0x00, 0xf0, 0xfe, 0x0e },
    { 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe },
    { 0x00, 0xfe, 0x1e, 0xe0, 0x00, 0xfe, 0x0e, 0xf0 },
    { 0x00, 0xfe, 0xe0, 0x1e, 0x00, 0xfe, 0xf0, 0x0e },
    { 0x00, 0xfe, 0xfe, 0x00, 0x00, 0xfe, 0xfe, 0x00 },
    { 0x1e, 0x00, 0x00, 0x1e, 0x0e, 0x00, 0x00, 0x0e },
    { 0x1e, 0x00, 0x1e, 0x00, 0x0e, 0x00, 0x0e, 0x00 },
    { 0x1e, 0x00, 0xe0, 0xfe, 0x0e, 0x00, 0xf0, 0xfe },
    { 0x1e, 0x00, 0xfe, 0xe0, 0x0e, 0x00, 0xfe, 0xf0 },
    { 0x1e, 0x1e, 0x00, 0x00, 0x0e, 0x0e, 0x00, 0x00 },
    { 0x1e, 0x1e, 0x1e, 0x1e, 0x0e, 0x0e, 0x0e, 0x0e },
    { 0x1e, 0x1e, 0xe0, 0xe0, 0x0e, 0x0e, 0xf0, 0xf0 },
    { 0x1e, 0x1e, 0xfe, 0xfe, 0x0e, 0x0e, 0xfe, 0xfe },
    { 0x1e, 0xe0, 0x00, 0xfe, 0x0e, 0xf0, 0x00, 0xfe },
    { 0x1e, 0xe0, 0x1e, 0xe0, 0x0e, 0xf0, 0x0e, 0xf0 },
    { 0x1e, 0xe0, 0xe0, 0x1e, 0x0e, 0xf0, 0xf0, 0x0e },
    { 0x1e, 0xe0, 0xfe, 0x00, 0x0e, 0xf0, 0xfe, 0x00 },
    { 0x1e, 0xfe, 0x00, 0xe0, 0x0e, 0xfe, 0x00, 0xf0 },
    { 0x1e, 0xfe, 0x1e, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe },
    { 0x1e, 0xfe, 0xe0, 0x00, 0x0e, 0xfe, 0xf0, 0x00 },
    { 0x1e, 0xfe, 0xfe, 0x1e, 0x0e, 0xfe, 0xfe, 0x0e },
    { 0xe0, 0x00, 0x00, 0xe0, 0xf0, 0x00, 0x00, 0xf0 },
    { 0xe0, 0x00, 0x1e, 0xfe, 0xf0, 0x00, 0x0e, 0xfe },
    { 0xe0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf0, 0x00 },
    { 0xe0, 0x00, 0xfe, 0x1e, 0xf0, 0x00, 0xfe, 0x0e },
    { 0xe0, 0x1e, 0x00, 0xfe, 0xf0, 0x0e, 0x00, 0xfe },
    { 0xe0, 0x1e, 0x1e, 0xe0, 0xf0, 0x0e, 0x0e, 0xf0 },
    { 0xe0, 0x1e, 0xe0, 0x1e, 0xf0, 0x0e, 0xf0, 0x0e },
    { 0xe0, 0x1e, 0xfe, 0x00, 0xf0, 0x0e, 0xfe, 0x00 },
    { 0xe0, 0xe0, 0x00, 0x00, 0xf0, 0xf0, 0x00, 0x00 },
    { 0xe0, 0xe0, 0x1e, 0x1e, 0xf0, 0xf0, 0x0e, 0x0e },
    { 0xe0, 0xe0, 0xe0, 0xe0, 0xf0, 0xf0, 0xf0, 0xf0 },
    { 0xe0, 0xe0, 0xfe, 0xfe, 0xf0, 0xf0, 0xfe, 0xfe },
    { 0xe0, 0xfe, 0x00, 0x1e, 0xf0, 0xfe, 0x00, 0x0e },
    { 0xe0, 0xfe, 0x1e, 0x00, 0xf0, 0xfe, 0x0e, 0x00 },
    { 0xe0, 0xfe, 0xe0, 0xfe, 0xf0, 0xfe, 0xf0, 0xfe },
    { 0xe0, 0xfe, 0xfe, 0xe0, 0xf0, 0xfe, 0xfe, 0xf0 },
    { 0xfe, 0x00, 0x00, 0xfe, 0xfe, 0x00, 0x00, 0xfe },
    { 0xfe, 0x00, 0x1e, 0xe0, 0xfe, 0x00, 0x0e, 0xf0 },
    { 0xfe, 0x00, 0xe0, 0x1e, 0xfe, 0x00, 0xf0, 0x0e },
    { 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00 },
    { 0xfe, 0x1e, 0x00, 0xe0, 0xfe, 0x0e, 0x00, 0xf0 },
    { 0xfe, 0x1e, 0x1e, 0xfe, 0xfe, 0x0e, 0x0e, 0xfe },
    { 0xfe, 0x1e, 0xe0, 0x00, 0xfe, 0x0e, 0xf0, 0x00 },
    { 0xfe, 0x1e, 0xfe, 0x1e, 0xfe, 0x0e, 0xfe, 0x0e },
    { 0xfe, 0xe0, 0x00, 0x1e, 0xfe, 0xf0, 0x00, 0x0e },
    { 0xfe, 0xe0, 0x1e, 0x00, 0xfe, 0xf0, 0x0e, 0x00 },
    { 0xfe, 0xe0, 0xe0, 0xfe, 0xfe, 0xf0, 0xf0, 0xfe },
    { 0xfe, 0xe0, 0xfe, 0xe0, 0xfe, 0xf0, 0xfe, 0xf0 },
    { 0xfe, 0xfe, 0x00, 0x00, 0xfe, 0xfe, 0x00, 0x00 },
    { 0xfe, 0xfe, 0x1e, 0x1e, 0xfe, 0xfe, 0x0e, 0x0e },
    { 0xfe, 0xfe, 0xe0, 0xe0, 0xfe, 0xfe, 0xf0, 0xf0 },
    { 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe }
};

#  endif

#endif

#define USED_DES_KEY_BITS CONST64(0xfefefefefefefefe)

BOOL __stdcall TC_DESKeyIsWeak(const void *pKey)
{
    const unsigned __int64 *pullKey;
    const unsigned __int64 *pullWeakKey;
    const unsigned __int64 *pullWeakKeyEnd;


    pullKey        = (const unsigned __int64 *) pKey;
    pullWeakKey    = (const unsigned __int64 *) pWeakDESKeys;
    pullWeakKeyEnd = pullWeakKey + WEAK_DES_KEYS;

    for (; pullWeakKey != pullWeakKeyEnd; pullWeakKey++) {
        if (((*pullKey ^ *pullWeakKey) & USED_DES_KEY_BITS) == 0) {
            return TRUE;
        }
    }

    return FALSE;
}

BOOL __stdcall TC_TripleDESKeyIsWeak(const void *pKey)
{
    const unsigned __int64 *pullK1;
    const unsigned __int64 *pullK2;
    const unsigned __int64 *pullK3;

    pullK1 = (const unsigned __int64 *) pKey;
    pullK2 = pullK1 + 1;
    pullK3 = pullK2 + 1;

    if (((*pullK1 ^ *pullK2) & USED_DES_KEY_BITS) == 0 ||
      ((*pullK2 ^ *pullK3) & USED_DES_KEY_BITS) == 0) {
        /* effective key size: 56 bits */
        return TRUE;
    }

    if (TC_DESKeyIsWeak(pullK1)) {
        return TRUE;
    }
    if (TC_DESKeyIsWeak(pullK2)) {
        return TRUE;
    }
    if (TC_DESKeyIsWeak(pullK3)) {
        return TRUE;
    }

    return FALSE;
}

#endif
