/*
    Project: TCTEMP
    Version: V1.7
    File   : tc_entropy.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tc_priv.h"

/*
 * Pool filling method taken from TrueCrypt keyfile applying algorithm
 */

void __stdcall TC_InitEntropyPool(PTC_ENTROPY_POOL pep, size_t cbPoolSize)
{
    TC_ASSERT(pep);
    TC_ASSERT((cbPoolSize & 3) == 0);

    if (cbPoolSize == 0) {
        cbPoolSize = sizeof(pep->rgbPool);
    }

    pep->pbWritePos = pep->rgbPool;
    pep->pbPoolEnd  = pep->rgbPool + cbPoolSize;
}

void __stdcall TC_FillEntropyPool(PTC_ENTROPY_POOL pep,
  const void *pData, size_t cbDataSize)
{
    const unsigned __int8 *pbData;
    const unsigned __int8 *pbDataEnd;

    TC_ASSERT(pep);
    TC_ASSERT(pData);

    pbData    = (const unsigned __int8 *) pData;
    pbDataEnd = pbData + cbDataSize;

    for (; pbData != pbDataEnd; pbData++) {
        UpdateCRC(pep->dwCRC, *pbData);
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[3];
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[2];
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[1];
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[0];
        if (pep->pbWritePos == pep->pbPoolEnd) {
            pep->pbWritePos = pep->rgbPool;
        }
    }
}
