/*
    Project: TCTEMP
    Version: V1.7
    File   : na_init.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "na_priv.h"

//#define DO_CLEAN_UP

NA_GLOBAL_PROCESS_INFO g_pi =
{
    NULL, NULL
};

#define GetNextAlignedPointer(p, a) \
  (PVOID) ( (((size_t)(p)) + ((a)-1)) & ~(size_t)((a)-1) )

BOOL NAAPI NaInitialize(PVOID *ppCtx, SIZE_T cbCtxSize, SIZE_T cbAlignment)
{
    if (g_pi.ppi) {
        return FALSE;
    }

    g_pi.hHeap = RtlCreateHeap(HEAP_GROWABLE, NULL, 0, 0, NULL, NULL);
    g_pi.ppi = (PNA_PROCESS_INFO) RtlAllocateHeap(g_pi.hHeap,
      HEAP_ZERO_MEMORY, (sizeof(NA_PROCESS_INFO) - 1) + cbAlignment + cbCtxSize);
    if (g_pi.ppi == NULL) {
#if DEBUG
        UNICODE_STRING usText;
        PCWSTR pcwszText = L"\nCannot allocate memory for NA_PROCESS_INFO!\n";
        DbgPrint("%ls\n", pcwszText);
        RtlInitUnicodeString(&usText, pcwszText);
        NtDisplayString(&usText);
#endif
        return FALSE;
    }

    NaAssert(cbAlignment);

    if (ppCtx) {
        *ppCtx = GetNextAlignedPointer(g_pi.ppi->rgbUserData, cbAlignment);
    }

    return TRUE;
}

void NAAPI NaUninitialize(void)
{
#if DEBUG
    if (g_pi.ppi == NULL || g_pi.ppi->fWaitBeforeExiting)
    {
        NaSleep(5000);
    }
#endif

#ifdef DO_CLEAN_UP
    if (g_pi.ppi) {
        RtlFreeHeap(g_pi.hHeap, 0, g_pi.ppi);
        RtlDestroyHeap(g_pi.hHeap);
        /* g_pi.ppi = NULL; */
    }
#endif
}